@@ -40,7 +40,7 @@
 #include <uim/uim-helper.h>
 
 #include "Utils.h"
-
+#include "Debug.h"
 
 typedef struct _PreeditSegment
 {
@@ -136,6 +136,9 @@ void
 GetPreeditString(MUIMSessionHandle inHandle, UniCharPtr *outStr,
                  UInt32 *outLen, UInt32 *outCursorPos);
 
+void
+SegmentToHiliteRange(MUIMSessionHandle inHandle, TextRangeArrayPtr *inRange);
+
 void
 DumpString(const char *name, const char *str, int len);
 
