@@ -63,17 +63,32 @@ MUIMUpdateActiveInputArea(MUIMSessionHandle inHandle, Boolean inFix)
   UniCharPtr str = NULL;
   UInt32 len = 0, cursorPos = 0;
   SInt32 fixLen;
-  UInt32 i, segmentPosition = 0;
+  //UInt32 i, j, segmentPosition = 0;
 
   if (!inFix) {
     GetPreeditString(inHandle, &str, &len, &cursorPos);
 
-    //DEBUG_PRINT("MUIMUpdateActiveInputArea() [NoFix] fOldPreeditLen=%lu len=%lu cursorPos=%lu\n",
-    //            (*inHandle)->fOldPreeditLen, len, cursorPos);
+#if DEBUG_INPUT_EVENT
+    DEBUG_PRINT("MUIMUpdateActiveInputArea() inFix=%s fOldPreeditLen=%lu len=%lu cursorPos=%lu\n",
+                inFix ? "true" : "false",
+                (*inHandle)->fOldPreeditLen, len, cursorPos);
+#endif
+
+#if 0 // XXX
+    if (len == 0 && cursorPos == 0) {
+#if DEBUG_INPUT_EVENT
+      DEBUG_PRINT("MUIMUpdateActiveInputArea() does not send event (XXX)\n");
+#endif
+      free(str);
+      return noErr;
+    }
+#endif
 
     if ((*inHandle)->fOldPreedit != NULL &&
-        //!memcmp((*inHandle)->fOldPreedit, str, sizeof(UniChar) * (len + 1))
         (*inHandle)->fOldPreeditLen == 0 && len == 0) {
+#if DEBUG_INPUT_EVENT
+      DEBUG_PRINT("MUIMUpdateActiveInputArea() does not send event\n");
+#endif
       free(str);
       return noErr;
     }
@@ -86,17 +101,14 @@ MUIMUpdateActiveInputArea(MUIMSessionHandle inHandle, Boolean inFix)
     len = cursorPos = (*inHandle)->fFixLen;
     fixLen = len * sizeof(UniChar);
     str = (*inHandle)->fFixBuffer;
-    /*
-    str = (UniCharPtr) malloc(fixLen + sizeof(UniChar));
-    memcpy(str, (*inHandle)->fFixBuffer, fixLen);
-    str[len] = '\0';
-    */
   }
 
-  //DEBUG_PRINT("MUIMUpdateActiveInputArea() inFix=%s len=%lu cursorPos=%lu fixLen=%ld\n",
-  //            inFix ? "true" : "false",
-  //            len, cursorPos, fixLen);
+#if DEBUG_INPUT_EVENT
+  DEBUG_PRINT("MUIMUpdateActiveInputArea() inFix=%s fOldPreeditLen=%lu len=%lu cursorPos=%lu fixLen=%ld\n",
+              inFix ? "true" : "false",
+              (*inHandle)->fOldPreeditLen, len, cursorPos, fixLen);
   //DumpString("str", (char *) str, len * sizeof(UniChar));
+#endif
 
   hiliteRangePtr = nil;
   updateRangePtr = nil;
@@ -134,8 +146,10 @@ MUIMUpdateActiveInputArea(MUIMSessionHandle inHandle, Boolean inFix)
                               typeUnicodeText,
                               len * sizeof(UniChar), str);
 
+#if DEBUG_INPUT_EVENT
   //DEBUG_PRINT("MUIMUpdateActiveInputArea - kEventParamTextInputSendText size=%d\n",
   //            len * sizeof(UniChar));
+#endif
  
   // set the fix length
   if (error == noErr)
@@ -143,8 +157,10 @@ MUIMUpdateActiveInputArea(MUIMSessionHandle inHandle, Boolean inFix)
                               typeLongInteger,
                               sizeof(SInt32), &fixLen);
 
+#if DEBUG_INPUT_EVENT
   //DEBUG_PRINT("MUIMUpdateActiveInputArea - kEventParamTextInputSendFixLen size=%d val=%d\n",
   //            sizeof(SInt32), fixLen);
+#endif
  
   // set the update range
   if (error == noErr) {
@@ -154,9 +170,8 @@ MUIMUpdateActiveInputArea(MUIMSessionHandle inHandle, Boolean inFix)
     if (updateRangePtr) {
       updateRangePtr->fNumOfRanges = 2;
       updateRangePtr->fRange[0].fStart = 0;
-      updateRangePtr->fRange[0].fEnd =
-        (*inHandle)->fLastUpdateLength * sizeof(UniChar);
-      updateRangePtr->fRange[1].fHiliteStyle = 0;
+      updateRangePtr->fRange[0].fEnd = (*inHandle)->fLastUpdateLength * sizeof(UniChar);
+      updateRangePtr->fRange[0].fHiliteStyle = 0;
       updateRangePtr->fRange[1].fStart = 0;
       updateRangePtr->fRange[1].fEnd = len * sizeof(UniChar);
       updateRangePtr->fRange[1].fHiliteStyle = 0;
@@ -172,74 +187,46 @@ MUIMUpdateActiveInputArea(MUIMSessionHandle inHandle, Boolean inFix)
                               sizeof(short) + sizeof(TextRange) * 2,
                               updateRangePtr);
 
-  //DEBUG_PRINT("MUIMUpdateActiveInputArea - fRange[0].fEnd=%d fRange[1].fEnd=%d\n",
-  //            updateRangePtr->fRange[0].fEnd,
-  //            updateRangePtr->fRange[1].fEnd);
+#if DEBUG_INPUT_EVENT
+  DEBUG_PRINT("MUIMUpdateActiveInputArea() [UpdateRng] fRange[0].fStart=%d fEnd=%d fRange[1].fStart=%d fEnd=%d\n",
+              updateRangePtr->fRange[0].fStart, updateRangePtr->fRange[0].fEnd,
+              updateRangePtr->fRange[1].fStart, updateRangePtr->fRange[1].fEnd);
+#endif
 
   // set the hilite range
   if (error == noErr) {
-    hiliteRangePtr =
-      (TextRangeArrayPtr) NewPtrClear(sizeof(short) +
-                                      sizeof(TextRange) *
-                                      ((*inHandle)->fSegmentCount + 1));
-    if (hiliteRangePtr) {
-      hiliteRangePtr->fNumOfRanges = (*inHandle)->fSegmentCount + 1;
-      for (i = 0; i < (*inHandle)->fSegmentCount; i++) {
-        hiliteRangePtr->fRange[i].fStart = segmentPosition * sizeof(UniChar);
-        segmentPosition += (*inHandle)->fSegments[i].fLength;
-        hiliteRangePtr->fRange[i].fEnd = segmentPosition * sizeof(UniChar);
-        /*
-         * kCaretPosition = 1,
-         * kRawText = 2,
-         * kSelectedRawText = 3,
-         * kConvertedText = 4,
-         * kSelectedConvertedText = 5,
-         * kBlockFillText = 6,
-         * kOutlineText = 7,
-         * kSelectedText = 8
-         */
-        if ((*inHandle)->fSegments[i].fAttr & (UPreeditAttr_Reverse | UPreeditAttr_Cursor)) {
-          hiliteRangePtr->fRange[i].fHiliteStyle = kSelectedConvertedText;
-        } else if (((*inHandle)->fSegments[i].fLength == 0) ||
-                 ((*inHandle)->fSegments[i].fLength == len) ||
-                 ((i > 0) && hiliteRangePtr->fRange[i - 1].fHiliteStyle == kRawText) ||
-                 /* XXX hack aleart! */
-                 /* for Anthy */
-                 ((i == 0) && ((*inHandle)->fSegmentCount > 2) && ((len - (*inHandle)->fSegments[i].fLength) == 1)) ||
-                 ((i == 0) && ((*inHandle)->fSegmentCount > 2) && ((len - (*inHandle)->fSegments[i].fLength) == 2)) ||
-                 /* for SKK */
-                 ((i == 0) && ((*inHandle)->fSegmentCount == 4) && ((*inHandle)->fSegments[i].fLength == 1))) {
-          hiliteRangePtr->fRange[i].fHiliteStyle = kRawText;
-        } else {
-          hiliteRangePtr->fRange[i].fHiliteStyle = kConvertedText;
-        }
-      }
-      hiliteRangePtr->fRange[i].fStart = cursorPos * sizeof(UniChar);
-      hiliteRangePtr->fRange[i].fEnd = cursorPos * sizeof(UniChar);
-      hiliteRangePtr->fRange[i].fHiliteStyle = kCaretPosition;
-    }
-    else
+    SegmentToHiliteRange(inHandle, &hiliteRangePtr);
+    if (!hiliteRangePtr)
       error = memFullErr;
   }
+
+#if DEBUG_INPUT_EVENT
+  {
+    int i;
+    for (i = 0; i < hiliteRangePtr->fNumOfRanges; i++) {
+      DEBUG_PRINT("MUIMUpdateActiveInputArea() [HiliteRng] fRange[%d].fStart=%d fEnd=%d fHiliteStyle=%d\n",
+                  i, hiliteRangePtr->fRange[i].fStart, hiliteRangePtr->fRange[i].fEnd,
+                  hiliteRangePtr->fRange[i].fHiliteStyle);
+    }
+  }
+#endif
+
   if (error == noErr)
     error = SetEventParameter(event, kEventParamTextInputSendHiliteRng,
                               typeTextRangeArray,
-                              sizeof(short) + sizeof(TextRange) * (i + 1),
+                              sizeof(short) + sizeof(TextRange) * hiliteRangePtr->fNumOfRanges,
                               hiliteRangePtr);
 
-  //DEBUG_PRINT("MUIMUpdateActiveInputArea - fRange[0].fEnd=%d fRange[1].fStart=%d fRange[1].fEnd=%d\n",
-  //            hiliteRangePtr->fRange[0].fEnd,
-  //            hiliteRangePtr->fRange[0].fStart,
-  //            hiliteRangePtr->fRange[0].fEnd);
-
   pinRange.fStart = 0;
   pinRange.fEnd = len * sizeof(UniChar);
   if (error == noErr)
     error = SetEventParameter(event, kEventParamTextInputSendPinRng,
                               typeTextRange, sizeof(TextRange), &pinRange);
 
-  //DEBUG_PRINT("MUIMUpdateActiveInputArea - pinRange.fEnd=%d\n",
-  //            pinRange.fEnd);
+#if DEBUG_INPUT_EVENT
+  DEBUG_PRINT("MUIMUpdateActiveInputArea() pinRange.fStart=%d pinRange.fEnd=%d\n",
+              pinRange.fStart, pinRange.fEnd);
+#endif
 
   if (error == noErr)
     error = SendTextInputEvent(event);
@@ -247,8 +234,6 @@ MUIMUpdateActiveInputArea(MUIMSessionHandle inHandle, Boolean inFix)
   DisposePtr((Ptr) hiliteRangePtr);
   DisposePtr((Ptr) updateRangePtr);
 
-  //free(str);
-
   return error;
 }
 
